@@ -16,10 +16,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA  02111-1307, USA.
  */
 
 #include "config.h"
@@ -36,10 +36,18 @@
 #include <string.h>
 #define _(String) gettext(String)
 
-using namespace gcu;
-
-namespace gcu
+static void ReadValue (char const *source, GcuValue &value)
 {
+	char *tmp, *dot;
+	value.value = strtod (source, &tmp);
+	dot = strchr (source, '.');
+	value.prec = (dot)? tmp - dot - 1: 0;
+	value.delta = (*tmp == '(')? strtol (tmp + 1, NULL, 10): 0;
+}
+
+using namespace gcu;
+using namespace std;
+
 class EltTable
 {
 public:
@@ -56,10 +64,6 @@ private:
 	map <string, Element*> EltsMap;
 };
 
-} // namespace gcu
-
-using namespace std;
-
 EltTable Table;
 
 EltTable::EltTable()
@@ -71,9 +75,9 @@ EltTable::EltTable()
 	xmlDocPtr xml;
 	char* DefaultName;
 	char *lang = getenv ("LANG");
-	char *old_num_locale, *tmp, *num;
+	char *old_num_locale, *tmp, *num, *dot;
 	unsigned char Z;
-	if (!(xml = xmlParseFile (DATADIR"/gchemutils/elements.xml")))
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/elements.xml")))
 	{
 		g_error (_("Can't find and read elements.xml"));
 	}
@@ -96,6 +100,10 @@ EltTable::EltTable()
 			num = (char*) xmlGetProp (node, (xmlChar*) "max_bonds");
 			Elt->m_MaxBonds = atoi (num);
 			xmlFree (num);
+			num = (char*) xmlGetProp (node, (xmlChar*) "weight");
+			Elt->m_Weight = strtod (num, &tmp);
+			dot = strchr (num, '.');
+			Elt->m_WeightPrec = (dot)? tmp - dot - 1: 0;
 			child = node->children;
 			DefaultName = NULL;
 			while (child)
@@ -131,78 +139,11 @@ EltTable::EltTable()
 						Elt->m_DefaultColor[2] = strtod (tmp, NULL);
 						xmlFree (tmp);
 					}
-				} else if (!strcmp ((const char*) child->name, "electronegativity")) {
-					GcuElectronegativity* en = new GcuElectronegativity;
-					en->Z = Z;	//FIXME: is it really useful there?
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "scale");
-					if (tmp) {
-						en->scale = g_strdup (tmp);
-						xmlFree (tmp);
-					} else
-						en->scale = NULL;
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
-					if (tmp) {
-						en->value = strtod (tmp, NULL);
-						Elt->m_en.push_back (en);
-						xmlFree (tmp);
-					} else
-						delete en;	//without a value, the structure is useless and is discarded
-				} else if (!strcmp ((const char*) child->name, "radius")) {
-					GcuAtomicRadius* radius = new GcuAtomicRadius;
-					radius->Z = Z;	//FIXME: is it really useful there?
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "type");
-					if (!tmp ||
-						((!((!strcmp (tmp, "covalent")) && (radius->type = GCU_COVALENT))) &&
-						(!((!strcmp (tmp, "vdW")) && (radius->type = GCU_VAN_DER_WAALS))) &&
-						(!((!strcmp (tmp, "ionic")) && (radius->type = GCU_IONIC))) &&
-						(!((!strcmp (tmp, "metallic")) && (radius->type = GCU_METALLIC))) &&
-						(!((!strcmp (tmp, "atomic")) && ((radius->type = GCU_ATOMIC) || true))))) {
-						//invalid radius
-						delete radius;
-						if (tmp)
-							xmlFree (tmp);
-						continue;
-					}
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "scale");
-					if (tmp) {
-						radius->scale = g_strdup (tmp);
-						xmlFree (tmp);
-					} else
-						radius->scale = NULL;
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "charge");
-					if (tmp) {
-						radius->charge = strtol (tmp, NULL, 10);
-						xmlFree (tmp);
-					} else
-						radius->charge = 0;
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "cn");
-					if (tmp) {
-						radius->cn = strtol (tmp, NULL, 10);
-						xmlFree (tmp);
-					} else
-						radius->cn = -1;
-					tmp = (char*) xmlGetProp (child, (xmlChar*)"spin");
-					if ((!tmp) ||
-						(!((!strcmp (tmp, "low")) && (radius->spin = GCU_LOW_SPIN))) &&
-						(!((!strcmp (tmp, "high")) && (radius->spin = GCU_HIGH_SPIN))))
-						radius->spin = GCU_N_A_SPIN;
-					if (tmp)
-						xmlFree (tmp);
-					tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
-					if (tmp) {
-						radius->value = strtod (tmp, NULL);
-						Elt->m_radii.push_back (radius);
-						xmlFree (tmp);
-					} else
-						delete radius;
-				}
-				child = child->next;
+				}				child = child->next;
 			}
 			if ((Elt->name.length () == 0) && DefaultName) Elt->name = DefaultName;
 			if (DefaultName)
 				xmlFree (DefaultName);
-			Elt->m_en.push_back (NULL);
-			Elt->m_radii.push_back (NULL);
 			AddElement (Elt);
 		}
 		node = node->next;
@@ -336,26 +277,26 @@ Element::Element(int Z, const char* Symbol)
 
 Element::~Element()
 {
-	while (!m_radii.empty())
-	{
+	while (!m_radii.empty()) {
 		GcuAtomicRadius *radius = m_radii.back();
-		if (radius)
-		{
+		if (radius) {
 			if (radius->scale) g_free(radius->scale);
 			delete radius;
 		}
 		m_radii.pop_back();
 	}
-	while (!m_en.empty())
-	{
+	while (!m_en.empty()) {
 		GcuElectronegativity* en = m_en.back();
-		if (en)
-		{
+		if (en) {
 			if (en->scale) g_free(en->scale);
 			delete en;
 		}
 		m_en.pop_back();
 	}
+	while (!m_isotopes.empty ()) {
+		delete (m_isotopes.back ());
+		m_isotopes.pop_back ();
+	}
 }
 
 const gchar* Element::Symbol(gint Z)
@@ -444,3 +385,301 @@ const GcuElectronegativity** Element::GetElectronegativities()
 {
 	return (const GcuElectronegativity**) &m_en.front();
 }
+
+double Element::GetWeight (int Z, int &prec)
+{
+	Element* Elt = Table[Z];
+	return (Elt)? Elt->GetWeight(prec): 0.;
+}
+
+void Element::LoadRadii ()
+{
+	xmlDocPtr xml;
+	char *old_num_locale, *tmp, *num, *dot, *end;
+	unsigned char Z;
+	static bool loaded = false;
+	if (loaded)
+		return;
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/radii.xml")))
+	{
+		g_error (_("Can't find and read radii.xml"));
+	}
+	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+	xmlNode* node = xml->children, *child;
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Uncorrect file format: radii.xml"));
+	node = node->children;
+	Element* Elt;
+	while (node) {
+		if (strcmp ((const char*) node->name, "text"))
+		{
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Uncorrect file format: radii.xml"));
+			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
+			Elt = Table[Z = atoi (num)];
+			child = node->children;
+			while (child)
+			{
+				if (!strcmp ((const char*) child->name, "text")) {
+					child = child->next;
+					continue;
+				}
+				if (!strcmp ((const char*) child->name, "radius")) {
+					GcuAtomicRadius* radius = new GcuAtomicRadius;
+					radius->Z = Z;	//FIXME: is it really useful there?
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "type");
+					if (!tmp ||
+						((!((!strcmp (tmp, "covalent")) && (radius->type = GCU_COVALENT))) &&
+						(!((!strcmp (tmp, "vdW")) && (radius->type = GCU_VAN_DER_WAALS))) &&
+						(!((!strcmp (tmp, "ionic")) && (radius->type = GCU_IONIC))) &&
+						(!((!strcmp (tmp, "metallic")) && (radius->type = GCU_METALLIC))) &&
+						(!((!strcmp (tmp, "atomic")) && ((radius->type = GCU_ATOMIC) || true))))) {
+						//invalid radius
+						delete radius;
+						if (tmp)
+							xmlFree (tmp);
+						continue;
+					}
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "scale");
+					if (tmp) {
+						radius->scale = g_strdup (tmp);
+						xmlFree (tmp);
+					} else
+						radius->scale = NULL;
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "charge");
+					if (tmp) {
+						radius->charge = strtol (tmp, NULL, 10);
+						xmlFree (tmp);
+					} else
+						radius->charge = 0;
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "cn");
+					if (tmp) {
+						radius->cn = strtol (tmp, NULL, 10);
+						xmlFree (tmp);
+					} else
+						radius->cn = -1;
+					tmp = (char*) xmlGetProp (child, (xmlChar*)"spin");
+					if ((!tmp) ||
+						(!((!strcmp (tmp, "low")) && (radius->spin = GCU_LOW_SPIN))) &&
+						(!((!strcmp (tmp, "high")) && (radius->spin = GCU_HIGH_SPIN))))
+						radius->spin = GCU_N_A_SPIN;
+					if (tmp)
+						xmlFree (tmp);
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
+					if (tmp) {
+						radius->value.value = strtod (tmp, &end);
+						dot = strchr (tmp, '.');
+						radius->value.prec = (dot)? end - dot - 1: 0;
+						Elt->m_radii.push_back (radius);
+						xmlFree (tmp);
+					} else
+						delete radius;
+				} else
+					g_error ("Invalid radius node");
+				child = child->next;
+			}
+			Elt->m_radii.push_back (NULL);
+		}
+		node = node->next;
+	}
+	setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+	xmlFreeDoc (xml);
+	loaded = true;
+}
+
+void Element::LoadElectronicProps ()
+{
+	xmlDocPtr xml;
+	char *old_num_locale, *tmp, *num, *dot, *end;
+	unsigned char Z;
+	static bool loaded = false;
+	if (loaded)
+		return;
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/elecprops.xml")))
+	{
+		g_error (_("Can't find and read elecprops.xml"));
+	}
+	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+	xmlNode* node = xml->children, *child;
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Uncorrect file format: elecprops.xml"));
+	node = node->children;
+	Element* Elt;
+	while (node) {
+		if (strcmp ((const char*) node->name, "text")) {
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Uncorrect file format: elecprops.xml"));
+			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
+			Elt = Table[Z = atoi (num)];
+			child = node->children;
+			while (child) {
+				if (!strcmp ((const char*) child->name, "text")) {
+					child = child->next;
+					continue;
+				}
+				if (!strcmp ((const char*) child->name, "electronegativity")) {
+					GcuElectronegativity* en = new GcuElectronegativity;
+					en->Z = Z;	//FIXME: is it really useful there?
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "scale");
+					if (tmp) {
+						en->scale = g_strdup (tmp);
+						xmlFree (tmp);
+					} else
+						en->scale = NULL;
+					tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
+					if (tmp) {
+						en->value.value = strtod (tmp, &end);
+						dot = strchr (tmp, '.');
+						en->value.prec = (dot)? end - dot - 1: 0;
+						Elt->m_en.push_back (en);
+						xmlFree (tmp);
+					} else
+						delete en;	//without a value, the structure is useless and is discarded
+				} else
+					g_error ("Invalid property node");
+				child = child->next;
+			}
+			Elt->m_en.push_back (NULL);
+		}
+		node = node->next;
+	}
+	setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+	xmlFreeDoc (xml);
+	loaded = true;
+}
+
+void Element::LoadIsotopes ()
+{
+	xmlDocPtr xml;
+	char *old_num_locale, *tmp, *num;
+	unsigned char Z;
+	static bool loaded = false;
+	if (loaded)
+		return;
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/isotopes.xml")))
+	{
+		g_error (_("Can't find and read isotopes.xml"));
+	}
+	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+	xmlNode* node = xml->children, *child;
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Uncorrect file format: isotopes.xml"));
+	node = node->children;
+	Element *Elt;
+	Isotope *Is;
+	int minA, maxA, niso;
+	while (node) {
+		if (strcmp ((const char*) node->name, "text")) {
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Uncorrect file format: isotopes.xml"));
+			minA = maxA = niso = 0;
+			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
+			Elt = Table[Z = atoi (num)];
+			xmlFree (num);
+			if (Elt == NULL)	// This should not occur
+				continue;
+			child = node->children;
+			while (child) {
+				if (!strcmp ((const char*) child->name, "text")) {
+					child = child->next;
+					continue;
+				}
+				if (!strcmp ((const char*) child->name, "isotope")) {
+					Is = new Isotope ();
+					num = (char*) xmlGetProp (child, (xmlChar*) "A");
+					if (num) {
+						Is->A = strtol (num, NULL, 10);
+						xmlFree (num);
+					}
+					num = (char*) xmlGetProp (child, (xmlChar*) "weight");
+					if (num) {
+						ReadValue (num, Is->mass);
+						xmlFree (num);
+					}
+					num = (char*) xmlGetProp (child, (xmlChar*) "abundance");
+					if (num) {
+						ReadValue (num, Is->abundance);
+						xmlFree (num);
+						niso++;
+						if (minA == 0)
+							minA = maxA = Is->A;
+						else {
+							if (minA > Is->A)
+								minA = Is->A;
+							else if (maxA < Is->A)
+								maxA = Is->A;
+						}
+					}
+					Elt->m_isotopes.push_back (Is);
+				}
+				child = child->next;
+			}
+			if (minA > 0) {
+				IsotopicPattern *pattern = new IsotopicPattern (minA, maxA);
+				vector<Isotope*>::iterator i, iend = Elt->m_isotopes.end ();
+				for (i = Elt->m_isotopes.begin (); i != iend; i++) {
+					if ((*i)->abundance.value != 0.)
+						pattern->SetValue ((*i)->A, (*i)->abundance.value);
+				}
+				pattern->Normalize ();
+				niso = pattern->GetMonoNuclNb ();
+				i = Elt->m_isotopes.begin ();
+				while ((*i)->A != niso)
+					i++;
+				pattern->SetMonoMass ((*i)->mass.value);
+				Elt->m_patterns.push_back (pattern);
+			}
+		}
+		node = node->next;
+	}
+	setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+	xmlFreeDoc (xml);
+	loaded = true;
+}
+
+void Element::LoadAllData ()
+{
+	LoadRadii ();
+	LoadElectronicProps ();
+	LoadIsotopes ();
+}
+
+IsotopicPattern *Element::GetIsotopicPattern (unsigned natoms)
+{
+	if (m_patterns.size () == 0)
+		return NULL;
+	IsotopicPattern *pat, *pattern, *result = NULL;
+	if (natoms == 0)
+		return NULL;
+	int i = 1;
+	while ((natoms & 1) == 0) {
+		natoms >>= 1;
+		i++;
+	}
+	while (natoms) {
+		if (i == 1) {
+			result = m_patterns[0];
+			result->Ref ();
+		} else if (natoms & 1) {
+			while (m_patterns.size () < i) {
+				pat = m_patterns[m_patterns.size () - 1]->square ();
+				pattern = pat->Simplify ();
+				pat->Unref ();
+				m_patterns.push_back (pattern);
+			}
+			pattern = m_patterns[i - 1];
+			if (result) {
+				pat = result->multiply (*pattern);
+				result->Unref ();
+				result = pat->Simplify ();
+				pat->Unref ();
+			} else {
+				result = pattern;
+				result->Ref ();
+			}
+		}
+		natoms >>= 1;
+		i++;
+	}
+	return result;
+}
