@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * element.h 
  *
- * Copyright (C) 2002-2004
+ * Copyright (C) 2002-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -18,22 +18,25 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA  02111-1307, USA.
  */
 
 #ifndef GCU_ELEMENT_H
 #define GCU_ELEMENT_H
 
 #include <glib.h>
+#include <map>
 #include <string>
 #include <vector>
 #include "chemistry.h"
+#include "isotope.h"
 
 using namespace std;
 
+class EltTable;
 
 /*!\namespace gcu
 The namespace used for all C++ classes provided by the Gnome Chemistry Utils.
@@ -41,8 +44,6 @@ The namespace used for all C++ classes provided by the Gnome Chemistry Utils.
 namespace gcu
 {
 
-class EltTable;
-
 /*!\class Element gcu/element.h
 	Represents a chemical element.
 	This class has no public constructor or destructor. The instances are created by the framework
@@ -92,6 +93,13 @@ public:
 	*/
 	static Element* GetElement (const gchar* symbol);
 	/*!
+	@param Z: the atomic number of a chemical element.
+	@param prec: receive the number of significant digits.
+
+	@return the atomic mass of the element.
+	*/
+	double  GetWeight (int Z, int &prec);
+	/*!
 	@param radius: a pointer to a GcuAtomicRadius structure.
 	
 	Before calling this function, most fields in the GcuAtomicRadius structure must be filled:
@@ -129,6 +137,22 @@ public:
 	@return the maximum number of bonds an atom of the element can be involved in.
 	*/
 	static unsigned GetMaxBonds (gint Z);
+	/*!
+	Loads the atomic radii database.
+	*/
+	static void LoadRadii ();
+	/*!
+	Loads the atomic electronic properties database.
+	*/
+	static void LoadElectronicProps ();
+	/*!
+	Loads the isotopes database.
+	*/
+	static void LoadIsotopes ();
+	/*!
+	Loads all databases.
+	*/
+	static void LoadAllData ();
 
 	/*!
 	\return The atomic number of the chemical element.
@@ -188,10 +212,25 @@ public:
 	including d and f electrons.
 	*/
 	unsigned GetMaxValenceElectrons () {return m_maxve;}
+	/*!
+	@param prec: receive the number of significant digits.
+
+	@return the atomic mass of the element.
+	*/
+	double GetWeight (int& prec) {prec = m_WeightPrec; return m_Weight;}
+	/*!
+	@param natoms: atoms count.
+
+	@return the isotopic pattern correponding to a fragment containing n atoms of the
+	element.
+	*/
+	IsotopicPattern *GetIsotopicPattern (unsigned natoms);
 	
 private:
 	unsigned char m_Z, m_nve, m_tve, m_maxve;
 	char m_Symbol[4];
+	double m_Weight;
+	int m_WeightPrec;
 	char m_DefaultValence;
 	unsigned char m_MaxBonds;
 	bool m_BestSide;
@@ -199,6 +238,9 @@ private:
 	string name;
 	vector<GcuAtomicRadius*> m_radii;
 	vector<GcuElectronegativity*> m_en;
+	vector<Isotope*> m_isotopes;
+	vector<IsotopicPattern*> m_patterns;
+	map<string, string> names;
 };
 
 } // namespace gcu
