@@ -18,10 +18,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA  02111-1307, USA.
  */
 
 #ifndef GCU_OBJECT_H
@@ -426,9 +426,9 @@ It should not be called by a program; call Object::EmitSignal instead.
 	virtual bool OnSignal (SignalId Signal, Object *Child);
 
 /*!
-@param state: whether to bloc signals or not
+@param state: whether to block signals or not
 
-Blocks signals if state is true and unblocks if state is false.
+Blocks signals if State is true and unblocs if state is false.
 
 Since 0.4.2
 */
